@@ -21,11 +21,19 @@ from sigma.modifiers import (
     SigmaLessThanEqualModifier,
     SigmaGreaterThanModifier,
     SigmaGreaterThanEqualModifier,
+    SigmaNotEqualModifier,
     SigmaExpandModifier,
     SigmaWindowsDashModifier,
+    SigmaTimestampMinuteModifier,
+    SigmaTimestampHourModifier,
+    SigmaTimestampDayModifier,
+    SigmaTimestampWeekModifier,
+    SigmaTimestampMonthModifier,
+    SigmaTimestampYearModifier,
 )
 from sigma.rule import SigmaDetectionItem
 from sigma.types import (
+    CompareOperators,
     SigmaBool,
     SigmaCasedString,
     SigmaExists,
@@ -38,6 +46,9 @@ from sigma.types import (
     SigmaRegularExpression,
     SigmaCompareExpression,
     SigmaCIDRExpression,
+    SpecialChars,
+    TimestampPart,
+    SigmaTimestampPart,
 )
 from sigma.conditions import ConditionAND
 from sigma.exceptions import SigmaRuleLocation, SigmaTypeError, SigmaValueError
@@ -45,7 +56,7 @@ from sigma.exceptions import SigmaRuleLocation, SigmaTypeError, SigmaValueError
 
 @pytest.fixture
 def dummy_detection_item():
-    return SigmaDetectionItem(None, [], ["foobar"])
+    return SigmaDetectionItem(None, [], [SigmaString("foobar")])
 
 
 @pytest.fixture
@@ -67,12 +78,12 @@ def dummy_union_modifier(dummy_detection_item):
 
 
 @pytest.fixture
-def dummy_sequence_modifier(dummy_detection_item):
-    class DummySequenceModifier(SigmaModifier):
-        def modify(self, val: Sequence[SigmaString]) -> List[SigmaString]:
+def dummy_list_modifier(dummy_detection_item):
+    class DummyListModifier(SigmaModifier):
+        def modify(self, val: list[SigmaString]) -> list[SigmaString]:
             return [SigmaString("")]
 
-    return DummySequenceModifier(dummy_detection_item, [])
+    return DummyListModifier(dummy_detection_item, [])
 
 
 def test_typecheck_plain(dummy_plain_modifier):
@@ -88,8 +99,8 @@ def test_typecheck_plain_wrong_apply(dummy_plain_modifier):
         dummy_plain_modifier.apply(SigmaNumber(123))
 
 
-def test_typecheck_apply_list_input(dummy_sequence_modifier):
-    assert dummy_sequence_modifier.apply([SigmaString("foobar")]) == [SigmaString("")]
+def test_typecheck_apply_list_input(dummy_list_modifier):
+    assert dummy_list_modifier.apply([SigmaString("foobar")]) == [SigmaString("")]
 
 
 def test_typecheck_union(dummy_union_modifier):
@@ -100,12 +111,12 @@ def test_typecheck_union_wrong(dummy_union_modifier):
     assert not dummy_union_modifier.type_check(SigmaRegularExpression(".*"))
 
 
-def test_typecheck_sequence(dummy_sequence_modifier):
-    assert dummy_sequence_modifier.type_check([SigmaString("foobar")])
+def test_typecheck_sequence(dummy_list_modifier):
+    assert dummy_list_modifier.type_check([SigmaString("foobar")])
 
 
-def test_typecheck_sequence_wrong(dummy_sequence_modifier):
-    assert not dummy_sequence_modifier.type_check([SigmaNumber(123)])
+def test_typecheck_sequence_wrong(dummy_list_modifier):
+    assert not dummy_list_modifier.type_check([SigmaNumber(123)])
 
 
 def test_contains_nowildcards(dummy_detection_item):
@@ -388,25 +399,31 @@ def test_all(dummy_detection_item):
 def test_lt(dummy_detection_item):
     assert SigmaLessThanModifier(dummy_detection_item, []).modify(
         SigmaNumber(123)
-    ) == SigmaCompareExpression(SigmaNumber(123), SigmaCompareExpression.CompareOperators.LT)
+    ) == SigmaCompareExpression(SigmaNumber(123), CompareOperators.LT)
 
 
 def test_lte(dummy_detection_item):
     assert SigmaLessThanEqualModifier(dummy_detection_item, []).modify(
         SigmaNumber(123)
-    ) == SigmaCompareExpression(SigmaNumber(123), SigmaCompareExpression.CompareOperators.LTE)
+    ) == SigmaCompareExpression(SigmaNumber(123), CompareOperators.LTE)
 
 
 def test_gt(dummy_detection_item):
     assert SigmaGreaterThanModifier(dummy_detection_item, []).modify(
         SigmaNumber(123)
-    ) == SigmaCompareExpression(SigmaNumber(123), SigmaCompareExpression.CompareOperators.GT)
+    ) == SigmaCompareExpression(SigmaNumber(123), CompareOperators.GT)
 
 
 def test_gte(dummy_detection_item):
     assert SigmaGreaterThanEqualModifier(dummy_detection_item, []).modify(
         SigmaNumber(123)
-    ) == SigmaCompareExpression(SigmaNumber(123), SigmaCompareExpression.CompareOperators.GTE)
+    ) == SigmaCompareExpression(SigmaNumber(123), CompareOperators.GTE)
+
+
+def test_nqe(dummy_detection_item):
+    assert SigmaNotEqualModifier(dummy_detection_item, []).modify(
+        SigmaNumber(123)
+    ) == SigmaCompareExpression(SigmaNumber(123), CompareOperators.NEQ)
 
 
 def test_fieldref(dummy_detection_item):
@@ -415,6 +432,29 @@ def test_fieldref(dummy_detection_item):
     ) == SigmaFieldReference("field")
 
 
+def test_fieldref_contains(dummy_detection_item):
+    fieldref = SigmaFieldReferenceModifier(dummy_detection_item, []).modify(SigmaString("field"))
+    assert (
+        SigmaContainsModifier(dummy_detection_item, [SigmaFieldReferenceModifier]).modify(fieldref)
+    ) == SigmaFieldReference("field", True, True)
+
+
+def test_fieldref_startswith(dummy_detection_item):
+    fieldref = SigmaFieldReferenceModifier(dummy_detection_item, []).modify(SigmaString("field"))
+    assert (
+        SigmaStartswithModifier(dummy_detection_item, [SigmaFieldReferenceModifier]).modify(
+            fieldref
+        )
+    ) == SigmaFieldReference("field", True, False)
+
+
+def test_fieldref_endswith(dummy_detection_item):
+    fieldref = SigmaFieldReferenceModifier(dummy_detection_item, []).modify(SigmaString("field"))
+    assert (
+        SigmaEndswithModifier(dummy_detection_item, [SigmaFieldReferenceModifier]).modify(fieldref)
+    ) == SigmaFieldReference("field", False, True)
+
+
 def test_fieldref_wildcard(dummy_detection_item):
     with pytest.raises(SigmaValueError, match="must not contain wildcards"):
         SigmaFieldReferenceModifier(dummy_detection_item, []).modify(SigmaString("field*"))
@@ -452,11 +492,21 @@ def test_compare_string(dummy_detection_item):
 
 
 def test_expand(dummy_detection_item):
-    assert SigmaExpandModifier(dummy_detection_item, []).modify(SigmaString("test%var%test")).s == (
+    assert SigmaExpandModifier(dummy_detection_item, []).modify(SigmaString("test%var%test")).s == [
         "test",
         Placeholder("var"),
         "test",
-    )
+    ]
+
+
+def test_expand_re(dummy_detection_item):
+    assert SigmaExpandModifier(dummy_detection_item, []).modify(
+        SigmaRegularExpression("test%var%test")
+    ).regexp.s == [
+        "test",
+        Placeholder("var"),
+        "test",
+    ]
 
 
 def test_cidr(dummy_detection_item):
@@ -477,3 +527,39 @@ def test_cidr_invalid(dummy_detection_item):
         SigmaCIDRModifier(dummy_detection_item, [], SigmaRuleLocation("test.yml")).modify(
             SigmaString("192.168.1.1/24")
         )
+
+
+def test_timestamp_minute_modifier(dummy_detection_item):
+    assert SigmaTimestampMinuteModifier(dummy_detection_item, []).modify(
+        SigmaNumber(123)
+    ) == SigmaTimestampPart(TimestampPart.MINUTE, 123)
+
+
+def test_timestamp_hour_modifier(dummy_detection_item):
+    assert SigmaTimestampHourModifier(dummy_detection_item, []).modify(
+        SigmaNumber(123)
+    ) == SigmaTimestampPart(TimestampPart.HOUR, 123)
+
+
+def test_timestamp_day_modifier(dummy_detection_item):
+    assert SigmaTimestampDayModifier(dummy_detection_item, []).modify(
+        SigmaNumber(123)
+    ) == SigmaTimestampPart(TimestampPart.DAY, 123)
+
+
+def test_timestamp_week_modifier(dummy_detection_item):
+    assert SigmaTimestampWeekModifier(dummy_detection_item, []).modify(
+        SigmaNumber(123)
+    ) == SigmaTimestampPart(TimestampPart.WEEK, 123)
+
+
+def test_timestamp_month_modifier(dummy_detection_item):
+    assert SigmaTimestampMonthModifier(dummy_detection_item, []).modify(
+        SigmaNumber(123)
+    ) == SigmaTimestampPart(TimestampPart.MONTH, 123)
+
+
+def test_timestamp_year_modifier(dummy_detection_item):
+    assert SigmaTimestampYearModifier(dummy_detection_item, []).modify(
+        SigmaNumber(123)
+    ) == SigmaTimestampPart(TimestampPart.YEAR, 123)
