@@ -4,7 +4,13 @@ import re
 from textwrap import dedent
 from sigma.processing.condition_expressions import ConditionAND, ConditionIdentifier, ConditionNOT
 from sigma.processing.finalization import ConcatenateQueriesFinalizer, JSONFinalizer
-from sigma.processing.pipeline import ProcessingPipeline, ProcessingItem, QueryPostprocessingItem
+from sigma.processing.pipeline import (
+    ProcessingPipeline,
+    ProcessingItem,
+    QueryPostprocessingItem,
+    SigmaPipelineParsingError,
+)
+from sigma.processing.transformations import transformations
 from sigma.processing.conditions import (
     DetectionItemProcessingItemAppliedCondition,
     FieldNameProcessingCondition,
@@ -20,8 +26,7 @@ from sigma.processing.conditions import (
 from sigma.processing.postprocessing import EmbedQueryTransformation
 from sigma.processing.transformations import (
     SetStateTransformation,
-    transformations,
-    Transformation,
+    PreprocessingTransformation,
     FieldMappingTransformation,
     AddFieldnamePrefixTransformation,
     FieldFunctionTransformation,
@@ -35,7 +40,7 @@ from sigma.types import SigmaString
 class RuleConditionTrue(RuleProcessingCondition):
     dummy: str
 
-    def match(self, pipeline: ProcessingPipeline, rule: SigmaRule) -> bool:
+    def match(self, rule: SigmaRule) -> bool:
         return True
 
 
@@ -43,7 +48,7 @@ class RuleConditionTrue(RuleProcessingCondition):
 class RuleConditionFalse(RuleProcessingCondition):
     dummy: str
 
-    def match(self, pipeline: ProcessingPipeline, rule: SigmaRule) -> bool:
+    def match(self, rule: SigmaRule) -> bool:
         return False
 
 
@@ -51,7 +56,7 @@ class RuleConditionFalse(RuleProcessingCondition):
 class DetectionItemConditionTrue(DetectionItemProcessingCondition):
     dummy: str
 
-    def match(self, pipeline: ProcessingPipeline, detection_item: SigmaDetectionItem) -> bool:
+    def match(self, detection_item: SigmaDetectionItem) -> bool:
         return True
 
 
@@ -59,7 +64,7 @@ class DetectionItemConditionTrue(DetectionItemProcessingCondition):
 class DetectionItemConditionFalse(DetectionItemProcessingCondition):
     dummy: str
 
-    def match(self, pipeline: ProcessingPipeline, detection_item: SigmaDetectionItem) -> bool:
+    def match(self, detection_item: SigmaDetectionItem) -> bool:
         return False
 
 
@@ -67,7 +72,7 @@ class DetectionItemConditionFalse(DetectionItemProcessingCondition):
 class FieldNameConditionTrue(FieldNameProcessingCondition):
     dummy: str
 
-    def match_field_name(self, pipeline: ProcessingPipeline, field_name: str) -> bool:
+    def match_field_name(self, field_name: str) -> bool:
         return True
 
 
@@ -75,25 +80,42 @@ class FieldNameConditionTrue(FieldNameProcessingCondition):
 class FieldNameConditionFalse(FieldNameProcessingCondition):
     dummy: str
 
-    def match_field_name(self, pipeline: ProcessingPipeline, field_name: str) -> bool:
+    def match_field_name(self, field_name: str) -> bool:
         return False
 
 
 @dataclass
-class TransformationPrepend(Transformation):
+class FieldNameConditionTrue(FieldNameProcessingCondition):
+    dummy: str
+
+    def match_field_name(self, field_name: str) -> bool:
+        return True
+
+
+@dataclass
+class FieldNameConditionFalse(FieldNameProcessingCondition):
+    dummy: str
+
+    def match_field_name(self, field_name: str) -> bool:
+        return False
+
+
+@dataclass
+class TransformationPrepend(PreprocessingTransformation):
     s: str
 
-    def apply(self, pipeline: ProcessingPipeline, rule: SigmaRule) -> SigmaRule:
+    def apply(self, rule: SigmaRule) -> SigmaRule:
+        super().apply(rule)
         rule.title = self.s + rule.title
         return rule
 
 
 @dataclass
-class TransformationAppend(Transformation):
+class TransformationAppend(PreprocessingTransformation):
     s: str
 
-    def apply(self, pipeline: ProcessingPipeline, rule: SigmaRule) -> SigmaRule:
-        super().apply(pipeline, rule)
+    def apply(self, rule: SigmaRule) -> SigmaRule:
+        super().apply(rule)
         rule.title += self.s
         return rule
 
@@ -364,8 +386,14 @@ def test_processingitem_default_linking():
 
 def test_processingitem_fromdict_without_id(processing_item_dict, processing_item):
     del processing_item_dict["id"]
-    processing_item.identifier = None
-    assert ProcessingItem.from_dict(processing_item_dict) == processing_item
+    item1 = ProcessingItem.from_dict(processing_item_dict)
+    item2 = ProcessingItem.from_dict(processing_item_dict)
+
+    # Should generate deterministic identifiers when no ID is provided
+    assert item1.identifier is not None
+    assert item2.identifier is not None
+    assert item1.identifier == item2.identifier
+    assert len(item1.identifier) == 16  # SHA256 hash truncated to 16 chars
 
 
 def test_processingitem_fromdict_missing_condition_type():
@@ -470,19 +498,22 @@ def test_processingitem_fromdict_unknown_transformation_parameter():
         )
 
 
-def test_processingitem_apply(processing_item, dummy_processing_pipeline, sigma_rule):
-    applied = processing_item.apply(dummy_processing_pipeline, sigma_rule)
+def test_processingitem_apply(processing_item, sigma_rule):
+    applied = processing_item.apply(sigma_rule)
     assert applied and sigma_rule.title == "TestTest"
 
 
-def test_processingitem_apply_condition_expr(
-    processing_item_with_condition_expr, dummy_processing_pipeline, sigma_rule
-):
-    applied = processing_item_with_condition_expr.apply(dummy_processing_pipeline, sigma_rule)
+def test_processingitem_apply_condition_expr(processing_item_with_condition_expr, sigma_rule):
+    applied = processing_item_with_condition_expr.apply(sigma_rule)
     assert applied and sigma_rule.title == "TestTest"
 
 
-def test_processingitem_apply_notapplied_all_with_false(dummy_processing_pipeline, sigma_rule):
+def test_processingitem_apply_condition_expr(processing_item_with_condition_expr, sigma_rule):
+    applied = processing_item_with_condition_expr.apply(sigma_rule)
+    assert applied and sigma_rule.title == "TestTest"
+
+
+def test_processingitem_apply_notapplied_all_with_false(sigma_rule):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         rule_condition_linking=all,
@@ -491,11 +522,11 @@ def test_processingitem_apply_notapplied_all_with_false(dummy_processing_pipelin
             RuleConditionFalse(dummy="test-false"),
         ],
     )
-    applied = processing_item.apply(dummy_processing_pipeline, sigma_rule)
+    applied = processing_item.apply(sigma_rule)
     assert not applied and sigma_rule.title == "Test"
 
 
-def test_processingitem_apply_negated_true(dummy_processing_pipeline, sigma_rule):
+def test_processingitem_apply_negated_true(sigma_rule):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         rule_condition_negation=True,
@@ -503,11 +534,11 @@ def test_processingitem_apply_negated_true(dummy_processing_pipeline, sigma_rule
             RuleConditionTrue(dummy="test-true"),
         ],
     )
-    applied = processing_item.apply(dummy_processing_pipeline, sigma_rule)
+    applied = processing_item.apply(sigma_rule)
     assert not applied and sigma_rule.title == "Test"
 
 
-def test_processingitem_apply_negated_false(dummy_processing_pipeline, sigma_rule):
+def test_processingitem_apply_negated_false(sigma_rule):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         rule_condition_negation=True,
@@ -515,11 +546,11 @@ def test_processingitem_apply_negated_false(dummy_processing_pipeline, sigma_rul
             RuleConditionFalse(dummy="test-false"),
         ],
     )
-    applied = processing_item.apply(dummy_processing_pipeline, sigma_rule)
+    applied = processing_item.apply(sigma_rule)
     assert applied and sigma_rule.title == "TestTest"
 
 
-def test_processingitem_apply_notapplied_all_with_false(dummy_processing_pipeline, sigma_rule):
+def test_processingitem_apply_notapplied_all_with_false(sigma_rule):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         rule_condition_linking=all,
@@ -528,11 +559,11 @@ def test_processingitem_apply_notapplied_all_with_false(dummy_processing_pipelin
             RuleConditionFalse(dummy="test-false"),
         ],
     )
-    applied = processing_item.apply(dummy_processing_pipeline, sigma_rule)
+    applied = processing_item.apply(sigma_rule)
     assert not applied and sigma_rule.title == "Test"
 
 
-def test_processingitem_match_detection_item(dummy_processing_pipeline, detection_item):
+def test_processingitem_match_detection_item(detection_item):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         detection_item_condition_linking=any,
@@ -541,12 +572,10 @@ def test_processingitem_match_detection_item(dummy_processing_pipeline, detectio
             DetectionItemConditionFalse(dummy="test-false"),
         ],
     )
-    assert processing_item.match_detection_item(dummy_processing_pipeline, detection_item) == True
+    assert processing_item.match_detection_item(detection_item) == True
 
 
-def test_processingitem_match_detection_item_dict_detections(
-    dummy_processing_pipeline, detection_item
-):
+def test_processingitem_match_detection_item_dict_detections(detection_item):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         detection_item_conditions={
@@ -555,12 +584,22 @@ def test_processingitem_match_detection_item_dict_detections(
         },
         detection_item_condition_linking=any,
     )
-    assert processing_item.match_detection_item(dummy_processing_pipeline, detection_item) == True
+    assert processing_item.match_detection_item(detection_item) == True
 
 
-def test_processingitem_match_detection_item_all_with_false(
-    dummy_processing_pipeline, detection_item
-):
+def test_processingitem_match_detection_item_dict_detections(detection_item):
+    processing_item = ProcessingItem(
+        transformation=TransformationAppend(s="Test"),
+        detection_item_conditions={
+            "cond1": DetectionItemConditionTrue(dummy="test-true"),
+            "cond2": DetectionItemConditionFalse(dummy="test-false"),
+        },
+        detection_item_condition_linking=any,
+    )
+    assert processing_item.match_detection_item(detection_item) == True
+
+
+def test_processingitem_match_detection_item_all_with_false(detection_item):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         detection_item_condition_linking=all,
@@ -569,12 +608,10 @@ def test_processingitem_match_detection_item_all_with_false(
             DetectionItemConditionFalse(dummy="test-false"),
         ],
     )
-    assert processing_item.match_detection_item(dummy_processing_pipeline, detection_item) == False
+    assert processing_item.match_detection_item(detection_item) == False
 
 
-def test_processingitem_match_detection_item_any_without_true(
-    dummy_processing_pipeline, detection_item
-):
+def test_processingitem_match_detection_item_any_without_true(detection_item):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         detection_item_condition_linking=any,
@@ -583,12 +620,10 @@ def test_processingitem_match_detection_item_any_without_true(
             DetectionItemConditionFalse(dummy="test-false"),
         ],
     )
-    assert processing_item.match_detection_item(dummy_processing_pipeline, detection_item) == False
+    assert processing_item.match_detection_item(detection_item) == False
 
 
-def test_processingitem_match_detection_item_negated_true(
-    dummy_processing_pipeline, detection_item
-):
+def test_processingitem_match_detection_item_negated_true(detection_item):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         detection_item_condition_negation=True,
@@ -596,12 +631,10 @@ def test_processingitem_match_detection_item_negated_true(
             DetectionItemConditionTrue(dummy="test-true"),
         ],
     )
-    assert processing_item.match_detection_item(dummy_processing_pipeline, detection_item) == False
+    assert processing_item.match_detection_item(detection_item) == False
 
 
-def test_processingitem_match_detection_item_negated_false(
-    dummy_processing_pipeline, detection_item
-):
+def test_processingitem_match_detection_item_negated_false(detection_item):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         detection_item_condition_negation=True,
@@ -609,28 +642,10 @@ def test_processingitem_match_detection_item_negated_false(
             DetectionItemConditionFalse(dummy="test-false"),
         ],
     )
-    assert processing_item.match_detection_item(dummy_processing_pipeline, detection_item)
+    assert processing_item.match_detection_item(detection_item)
 
 
-def test_processingitem_match_detection_item_with_expression(
-    dummy_processing_pipeline, detection_item
-):
-    processing_item = ProcessingItem(
-        transformation=TransformationAppend(s="Test"),
-        detection_item_conditions={
-            "cond1": DetectionItemConditionTrue(dummy="test-true"),
-            "cond2": DetectionItemConditionFalse(dummy="test-false"),
-        },
-        detection_item_condition_expression=ConditionAND(
-            0, ConditionIdentifier(0, "cond1"), ConditionNOT(10, ConditionIdentifier(14, "cond2"))
-        ),
-    )
-    assert processing_item.match_detection_item(dummy_processing_pipeline, detection_item)
-
-
-def test_processingitem_match_detection_item_with_expression_false(
-    dummy_processing_pipeline, detection_item
-):
+def test_processingitem_match_detection_item_with_expression_false(detection_item):
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         detection_item_conditions={
@@ -641,10 +656,10 @@ def test_processingitem_match_detection_item_with_expression_false(
             0, ConditionIdentifier(0, "cond1"), ConditionNOT(10, ConditionIdentifier(14, "cond2"))
         ),
     )
-    assert not processing_item.match_detection_item(dummy_processing_pipeline, detection_item)
+    assert not processing_item.match_detection_item(detection_item)
 
 
-def test_processingitem_match_field_name(dummy_processing_pipeline):
+def test_processingitem_match_field_name():
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         field_name_conditions=[
@@ -653,10 +668,10 @@ def test_processingitem_match_field_name(dummy_processing_pipeline):
         ],
         field_name_condition_linking=any,
     )
-    assert processing_item.match_field_name(dummy_processing_pipeline, "field") == True
+    assert processing_item.match_field_name("field") == True
 
 
-def test_processingitem_match_field_name_dict_fields(dummy_processing_pipeline):
+def test_processingitem_match_field_name_dict_fields():
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         field_name_conditions={
@@ -665,10 +680,10 @@ def test_processingitem_match_field_name_dict_fields(dummy_processing_pipeline):
         },
         field_name_condition_linking=any,
     )
-    assert processing_item.match_field_name(dummy_processing_pipeline, "field") == True
+    assert processing_item.match_field_name("field") == True
 
 
-def test_processingitem_match_field_name_with_expression(dummy_processing_pipeline):
+def test_processingitem_match_field_name_with_expression():
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         field_name_conditions={
@@ -679,10 +694,10 @@ def test_processingitem_match_field_name_with_expression(dummy_processing_pipeli
             0, ConditionIdentifier(0, "cond1"), ConditionNOT(10, ConditionIdentifier(14, "cond2"))
         ),
     )
-    assert processing_item.match_field_name(dummy_processing_pipeline, "field") == True
+    assert processing_item.match_field_name("field") == True
 
 
-def test_processingitem_match_field_name_with_expression_false(dummy_processing_pipeline):
+def test_processingitem_match_field_name_with_expression_false():
     processing_item = ProcessingItem(
         transformation=TransformationAppend(s="Test"),
         field_name_conditions={
@@ -693,7 +708,7 @@ def test_processingitem_match_field_name_with_expression_false(dummy_processing_
             0, ConditionIdentifier(0, "cond1"), ConditionNOT(10, ConditionIdentifier(14, "cond2"))
         ),
     )
-    assert not processing_item.match_field_name(dummy_processing_pipeline, "field")
+    assert not processing_item.match_field_name("field")
 
 
 def test_processingitem_rule_condition_no_list_or_dict():
@@ -839,12 +854,36 @@ def test_postprocessingitem_fromdict(postprocessing_item_dict, postprocessing_it
     assert QueryPostprocessingItem.from_dict(postprocessing_item_dict) == postprocessing_item
 
 
-def test_postprocessingitem_apply(
-    postprocessing_item: QueryPostprocessingItem, dummy_processing_pipeline, sigma_rule
+def test_postprocessingitem_apply(postprocessing_item: QueryPostprocessingItem, sigma_rule):
+    postprocessing_item.apply(sigma_rule, "field=value") == "[ field=value ]"
+
+
+def test_postprocessingitem_apply_false_condition(
+    postprocessing_item: QueryPostprocessingItem, sigma_rule, monkeypatch
 ):
-    postprocessing_item.apply(
-        dummy_processing_pipeline, sigma_rule, "field=value"
-    ) == "[ field=value ]"
+    monkeypatch.setattr(postprocessing_item, "rule_conditions", [RuleConditionFalse(dummy="test")])
+    assert postprocessing_item.apply(sigma_rule, "field=value") == (
+        "field=value",
+        False,
+    )
+
+
+def test_postprocessingitem_condition_not_list():
+    with pytest.raises(SigmaTypeError, match="must be provided as list"):
+        QueryPostprocessingItem(
+            transformation=EmbedQueryTransformation(prefix="[ ", suffix=" ]"),
+            rule_conditions=RuleConditionTrue(dummy="test-true"),
+            identifier="test",
+        )
+
+
+def test_postprocessingitem_wrong_condition_type():
+    with pytest.raises(SigmaTypeError, match="is not a RuleProcessingCondition"):
+        QueryPostprocessingItem(
+            transformation=EmbedQueryTransformation(prefix="[ ", suffix=" ]"),
+            rule_conditions=[DetectionItemConditionTrue(dummy="test-true")],
+            identifier="test",
+        )
 
 
 def test_processingpipeline_fromdict(
@@ -946,6 +985,35 @@ def test_processingpipeline_fromyaml(
     )
 
 
+def test_processingpipeline_fromyaml_invalid(
+    processing_item_dict, processing_item, postprocessing_item, processing_pipeline_vars
+):
+    with pytest.raises(
+        SigmaPipelineParsingError, match="Error in parsing of a Sigma processing pipeline"
+    ):
+        ProcessingPipeline.from_yaml(
+            """
+                {not a yaml
+            """
+        )
+
+
+def test_processingpipeline_fromyaml_unknown(
+    processing_item_dict, processing_item, postprocessing_item, processing_pipeline_vars
+):
+    with pytest.raises(SigmaConfigurationError, match="Unkown keys \['transformation'\]"):
+        ProcessingPipeline.from_yaml(
+            """
+        name: unknown
+        priority: 10
+        transformation:
+        - id: test
+          type: test
+          method: test
+            """
+        )
+
+
 def test_processingpipeline_fromdict_error(processing_item_dict_with_error):
     with pytest.raises(SigmaConfigurationError, match="Error in processing rule 1:.*2"):
         ProcessingPipeline.from_dict(
@@ -1349,42 +1417,15 @@ def test_processingpipeline_invalid_concatenation_left():
         )
 
 
-@pytest.fixture(scope="module")
-def processing_pipeline_with_field_func_transform():
-    return ProcessingPipeline(
-        items=[
-            ProcessingItem(  # Field mappings
-                identifier="field_transform",
-                transformation=FieldFunctionTransformation(
-                    transform_func=lambda f: f.upper(),
-                    mapping={
-                        "fieldA": "mappedA",
-                    },
-                ),
-            ),
-        ]
+def test_processingitem_match_detection_item_with_field_name_condition_expression(detection_item):
+    processing_item = ProcessingItem(
+        transformation=TransformationAppend(s="Test"),
+        field_name_conditions={
+            "cond1": FieldNameConditionTrue(dummy="test-true"),
+            "cond2": FieldNameConditionFalse(dummy="test-false"),
+        },
+        field_name_condition_expression=ConditionAND(
+            0, ConditionIdentifier(0, "cond1"), ConditionNOT(10, ConditionIdentifier(14, "cond2"))
+        ),
     )
-
-
-def test_processingpipeline_field_name_transformation_in_field_list(
-    processing_pipeline_with_field_func_transform,
-):
-    rule = processing_pipeline_with_field_func_transform.apply(
-        SigmaRule.from_yaml(
-            f"""
-            title: Test
-            status: test
-            logsource:
-                category: test
-            detection:
-                sel:
-                    fieldA: valueA
-                    fieldB: valueB
-                condition: sel
-            fields:
-                - fieldA
-                - fieldB
-        """
-        )
-    )
-    assert rule.fields == ["mappedA", "FIELDB"]
+    assert processing_item.match_detection_item(detection_item) == True
