@@ -19,6 +19,9 @@ from sigma.types import (
     SigmaNumber,
     SigmaNull,
     SigmaRegularExpression,
+    SigmaTimestampPart,
+    TimestampPart,
+    SigmaCompareExpression,
 )
 from sigma.modifiers import (
     SigmaBase64Modifier,
@@ -55,13 +58,19 @@ def test_sigmalevel_comparation():
 
 
 def test_sigmalevel_comparation_invalid():
-    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaLevel .*"):
-        assert SigmaLevel.HIGH == "HIGH"
-        assert SigmaLevel.HIGH >= "LOW"
-        assert SigmaLevel.HIGH > "LOW"
-        assert SigmaLevel.HIGH != "LOW"
-        assert SigmaLevel.LOW <= "HIGH"
-        assert SigmaLevel.LOW < "HIGH"
+    # Equality and inequality comparisons should return False/True for incompatible types
+    assert (SigmaLevel.HIGH == "HIGH") is False
+    assert (SigmaLevel.HIGH != "LOW") is True
+
+    # Ordering comparisons should still raise exceptions for incompatible types
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaLevel"):
+        SigmaLevel.HIGH >= "LOW"
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaLevel"):
+        SigmaLevel.HIGH > "LOW"
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaLevel"):
+        SigmaLevel.LOW <= "HIGH"
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaLevel"):
+        SigmaLevel.LOW < "HIGH"
 
 
 def test_sigmastatus_str():
@@ -78,13 +87,35 @@ def test_sigmastatus_comparation():
 
 
 def test_sigmastatus_comparation_invalid():
-    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaStatus .*"):
-        assert SigmaStatus.STABLE == "STABLE"
-        assert SigmaStatus.STABLE >= "EXPERIMENTAL"
-        assert SigmaStatus.STABLE > "EXPERIMENTAL"
-        assert SigmaStatus.STABLE != "EXPERIMENTAL"
-        assert SigmaStatus.EXPERIMENTAL <= "STABLE"
-        assert SigmaStatus.EXPERIMENTAL < "STABLE"
+    # Equality and inequality comparisons should return False/True for incompatible types
+    assert (SigmaStatus.STABLE == "STABLE") is False
+    assert (SigmaStatus.STABLE != "EXPERIMENTAL") is True
+
+    # Ordering comparisons should still raise exceptions for incompatible types
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaStatus"):
+        SigmaStatus.STABLE >= "EXPERIMENTAL"
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaStatus"):
+        SigmaStatus.STABLE > "EXPERIMENTAL"
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaStatus"):
+        SigmaStatus.EXPERIMENTAL <= "STABLE"
+    with pytest.raises(sigma_exceptions.SigmaTypeError, match="Must be a SigmaStatus"):
+        SigmaStatus.EXPERIMENTAL < "STABLE"
+
+
+def test_sigmastatus_comparation_with_none():
+    """Test that SigmaStatus can be compared with None without raising exceptions."""
+    assert (None == SigmaStatus.STABLE) is False
+    assert (SigmaStatus.STABLE == None) is False
+    assert (None != SigmaStatus.STABLE) is True
+    assert (SigmaStatus.STABLE != None) is True
+
+
+def test_sigmalevel_comparation_with_none():
+    """Test that SigmaLevel can be compared with None without raising exceptions."""
+    assert (None == SigmaLevel.HIGH) is False
+    assert (SigmaLevel.HIGH == None) is False
+    assert (None != SigmaLevel.HIGH) is True
+    assert (SigmaLevel.HIGH != None) is True
 
 
 ### SigmaRuleTag tests ###
@@ -223,7 +254,7 @@ def test_sigmalogsource_not_in():
 
 
 def test_sigmalogsource_in_invalid():
-    with pytest.raises(TypeError):
+    with pytest.raises(sigma_exceptions.SigmaTypeError):
         assert 123 in SigmaLogSource("category", "product", "service")
 
 
@@ -235,20 +266,6 @@ def test_sigmadetectionitem_keyword_single():
     )
 
 
-def test_sigmadetectionitem_value_cleanup_single():
-    """Single value cleanup."""
-    assert SigmaDetectionItem(None, [], "value") == SigmaDetectionItem(
-        None, [], [SigmaString("value")]
-    )
-
-
-def test_sigmadetectionitem_value_cleanup_multi():
-    """Multiple value cleanup."""
-    assert SigmaDetectionItem(None, [], ["value", 123]) == SigmaDetectionItem(
-        None, [], [SigmaString("value"), SigmaNumber(123)]
-    )
-
-
 def test_sigmadetectionitem_keyword_single_to_plain():
     """Single keyword detection."""
     assert SigmaDetectionItem(None, [], [SigmaString("value*")]).to_plain() == "value*"
@@ -438,7 +455,9 @@ def test_sigmadetectionitem_key_value_modifiers_invalid_re():
 
 
 def test_sigmadetectionitem_fromvalue():
-    SigmaDetectionItem.from_value("test") == SigmaDetectionItem(None, [], [SigmaString("test")])
+    assert SigmaDetectionItem.from_value("test") == SigmaDetectionItem(
+        None, [], [SigmaString("test")]
+    )
 
 
 def test_sigmadetectionitem_processing_item_tracking(processing_item):
@@ -448,6 +467,22 @@ def test_sigmadetectionitem_processing_item_tracking(processing_item):
     assert detection_item.was_processed_by("test")
 
 
+def test_sigmadetectionitem_invalid_value_type():
+    """Test that SigmaDetectionItem raises an error if value is not a list."""
+    with pytest.raises(
+        sigma_exceptions.SigmaTypeError, match="Value must be a list of SigmaType instances"
+    ):
+        SigmaDetectionItem(field="key", modifiers=[], value="invalid_value")
+
+
+def test_sigmadetectionitem_invalid_value_elements():
+    """Test that SigmaDetectionItem raises an error if value contains non-SigmaType elements."""
+    with pytest.raises(
+        sigma_exceptions.SigmaTypeError, match="Value must be a list of SigmaType instances"
+    ):
+        SigmaDetectionItem(field="key", modifiers=[], value=["valid", 123, object()])
+
+
 ### SigmaDetection tests ###
 
 
@@ -566,16 +601,16 @@ def test_sigmadetections_fromdict():
 def test_sigmadetections_to_dict_single_condition():
     assert SigmaDetections(
         detections={
-            "test": SigmaDetection([SigmaDetectionItem("field", [], SigmaString("value"))])
+            "test": SigmaDetection([SigmaDetectionItem("field", [], [SigmaString("value")])])
         },
         condition=["test"],
     ).to_dict() == {"test": {"field": "value"}, "condition": "test"}
 
 
-def test_sigmadetections_to_dict_single_condition():
+def test_sigmadetections_to_dict_double_condition():
     assert SigmaDetections(
         detections={
-            "test": SigmaDetection([SigmaDetectionItem("field", [], SigmaString("value"))])
+            "test": SigmaDetection([SigmaDetectionItem("field", [], [SigmaString("value")])])
         },
         condition=["test", "all of them"],
     ).to_dict() == {"test": {"field": "value"}, "condition": ["test", "all of them"]}
@@ -620,6 +655,78 @@ def test_sigmadetections_fromdict_no_condition():
         )
 
 
+def test_sigmadetections_empty_condition_list():
+    with pytest.raises(
+        sigma_exceptions.SigmaConditionError, match="at least one condition.*test.yml"
+    ):
+        SigmaDetections(
+            detections={
+                "selection": SigmaDetection([SigmaDetectionItem("key", [], [SigmaString("value")])])
+            },
+            condition=[],
+            source=sigma_exceptions.SigmaRuleLocation("test.yml"),
+        )
+
+
+def test_sigmadetections_none_condition():
+    with pytest.raises(
+        sigma_exceptions.SigmaConditionError, match="at least one condition.*test.yml"
+    ):
+        SigmaDetections(
+            detections={
+                "selection": SigmaDetection([SigmaDetectionItem("key", [], [SigmaString("value")])])
+            },
+            condition=None,
+            source=sigma_exceptions.SigmaRuleLocation("test.yml"),
+        )
+
+
+def test_sigmadetections_from_to_dict_inversity_with_list_of_dicts():
+    detections_dict = {
+        "selection": [
+            {
+                "fieldA": ["value1", "value2"],
+                "fieldB": ["value3", "value4"],
+            },
+            {
+                "fieldA": "value5",
+                "fieldC": "value6",
+            },
+        ],
+        "condition": "selection",
+    }
+
+    detections = SigmaDetections.from_dict(detections_dict)
+    assert detections.to_dict() == detections_dict
+
+
+def test_sigmadetections_from_to_dict_inversity_with_dict_of_lists():
+    detections_dict = {
+        "selection": {
+            "fieldA": ["value1", "value2", "value3"],
+            "fieldB": ["value4", "value5"],
+        },
+        "condition": "selection",
+    }
+
+    detections = SigmaDetections.from_dict(detections_dict)
+    assert detections.to_dict() == detections_dict
+
+
+def test_sigmadetections_from_to_dict_inversity_with_dict_of_values():
+    detections_dict = {
+        "selection": {
+            "fieldA": "value1",
+            "fieldB": 123,
+            "fieldC": True,
+        },
+        "condition": "selection",
+    }
+
+    detections = SigmaDetections.from_dict(detections_dict)
+    assert detections.to_dict() == detections_dict
+
+
 def test_detectionitem_all_modified_key_plain_values_postprocess():
     """
     Test if postprocessed condition result of an all-modified field-bound value list results in an
@@ -671,16 +778,67 @@ def test_detectionitem_all_modified_key_special_values_postprocess():
     )
 
 
+def test_sigmadetection_from_definition_mapping():
+    """Test if SigmaDetection can be created from a definition mapping."""
+    definition_mapping = {
+        "field1": ["value1", "value2"],
+        "field2": "value3",
+        "field3|re": "regex.*pattern",
+    }
+    detection = SigmaDetection.from_definition(definition_mapping)
+    assert detection.detection_items == [
+        SigmaDetectionItem("field1", [], [SigmaString("value1"), SigmaString("value2")]),
+        SigmaDetectionItem("field2", [], [SigmaString("value3")]),
+        SigmaDetectionItem(
+            "field3",
+            [SigmaRegularExpressionModifier],
+            [SigmaRegularExpression("regex.*pattern")],
+            auto_modifiers=False,
+        ),
+    ]
+
+
+def test_sigmadetection_from_definition_plain_value():
+    detection = SigmaDetection.from_definition("keyword")
+    assert detection.detection_items == [SigmaDetectionItem(None, [], [SigmaString("keyword")])]
+
+
+def test_sigmadetection_from_definition_list_of_values():
+    detection = SigmaDetection.from_definition(["value1", "value2"])
+    assert detection.detection_items == [
+        SigmaDetectionItem(None, [], [SigmaString("value1"), SigmaString("value2")])
+    ]
+
+
+def test_sigmadetection_from_definition_list_of_dicts():
+    detection = SigmaDetection.from_definition(
+        [
+            {"field1": "value1"},
+            {"field2": "value2"},
+        ]
+    )
+    assert detection.detection_items == [
+        SigmaDetection([SigmaDetectionItem("field1", [], [SigmaString("value1")])]),
+        SigmaDetection([SigmaDetectionItem("field2", [], [SigmaString("value2")])]),
+    ]
+
+
+def test_sigmadetection_from_definition_unsupported_type():
+    """Test if an unsupported type raises an error."""
+    with pytest.raises(sigma_exceptions.SigmaDetectionError, match="Unsupported.*type"):
+        SigmaDetection.from_definition(object())
+
+
 def test_sigmadetection_processing_item_tracking(processing_item):
     """Key-value detection with one value."""
     detection = SigmaDetection(
         [
-            SigmaDetectionItem("field1", [], SigmaString("value1")),
-            SigmaDetectionItem("field2", [], SigmaString("value2")),
+            SigmaDetectionItem("field1", [], [SigmaString("value1")]),
+            SigmaDetectionItem("field2", [], [SigmaString("value2")]),
             SigmaDetection(
                 [
-                    SigmaDetectionItem("field3", [], SigmaString("value3")),
-                    SigmaDetectionItem("field4", [], SigmaString("value4")),
+                    SigmaDetectionItem("field3", [], [SigmaString("value3")]),
+                    SigmaDetectionItem("field4", [], [SigmaString("value4")]),
                 ]
             ),
         ]
@@ -705,15 +863,15 @@ def test_sigmadetection_processing_item_tracking(processing_item):
 
 def test_sigmadetection_single_to_plain():
     assert SigmaDetection(
-        detection_items=[SigmaDetectionItem("field", [], SigmaString("value"))]
+        detection_items=[SigmaDetectionItem("field", [], [SigmaString("value")])]
     ).to_plain() == {"field": "value"}
 
 
 def test_sigmadetection_multi_dict_to_plain():
     assert SigmaDetection(
         detection_items=[
-            SigmaDetectionItem("field1", [], SigmaString("value1")),
-            SigmaDetectionItem("field2", [], SigmaString("value2")),
+            SigmaDetectionItem("field1", [], [SigmaString("value1")]),
+            SigmaDetectionItem("field2", [], [SigmaString("value2")]),
         ]
     ).to_plain() == {
         "field1": "value1",
@@ -725,8 +883,8 @@ def test_sigmadetection_multi_dict_to_plain_key_collision():
     """Two field names exist in distinct detection items and have to be merged."""
     assert SigmaDetection(
         detection_items=[
-            SigmaDetectionItem("field", [], SigmaString("value1")),
-            SigmaDetectionItem("field", [], SigmaString("value2")),
+            SigmaDetectionItem("field", [], [SigmaString("value1")]),
+            SigmaDetectionItem("field", [], [SigmaString("value2")]),
         ]
     ).to_plain() == {"field|all": ["value1", "value2"]}
 
@@ -735,8 +893,8 @@ def test_sigmadetection_multi_dict_to_plain_all_key_collision():
     """Two field names exist in distinct detection items and have to be merged."""
     assert SigmaDetection(
         detection_items=[
-            SigmaDetectionItem("field|all", [], SigmaString("value1")),
-            SigmaDetectionItem("field|all", [], SigmaString("value2")),
+            SigmaDetectionItem("field|all", [], [SigmaString("value1")]),
+            SigmaDetectionItem("field|all", [], [SigmaString("value2")]),
         ]
     ).to_plain() == {"field|all": ["value1", "value2"]}
 
@@ -754,7 +912,7 @@ def test_sigmadetection_multi_dict_to_plain_all_key_collision_list_values():
 def test_sigmadetection_multi_dict_to_plain_key_collision_all():
     assert SigmaDetection(
         detection_items=[
-            SigmaDetectionItem("field", [], SigmaString("value1")),
+            SigmaDetectionItem("field", [], [SigmaString("value1")]),
             SigmaDetectionItem("field|all", [], [SigmaString("value2"), SigmaString("value3")]),
         ]
     ).to_plain() == {
@@ -801,7 +959,7 @@ def test_sigmadetection_multi_dict_to_plain_key_collision_all_2single():
 def test_sigmadetection_lists_and_plain_to_plain():
     assert SigmaDetection(
         detection_items=[
-            SigmaDetectionItem(None, [], SigmaString("value1")),
+            SigmaDetectionItem(None, [], [SigmaString("value1")]),
             SigmaDetectionItem(None, [], [SigmaString("value2"), SigmaString("value3")]),
             SigmaDetectionItem(None, [], [SigmaString("value4"), SigmaString("value5")]),
         ]
@@ -812,13 +970,35 @@ def test_sigmadetection_dict_and_keyword_to_plain():
     with pytest.raises(sigma_exceptions.SigmaValueError, match="Can't convert detection.*test.yml"):
         SigmaDetection(
             detection_items=[
-                SigmaDetectionItem("field", [], SigmaString("value")),
-                SigmaDetectionItem(None, [], SigmaString("keyword")),
+                SigmaDetectionItem("field", [], [SigmaString("value")]),
+                SigmaDetectionItem(None, [], [SigmaString("keyword")]),
             ],
             source=sigma_exceptions.SigmaRuleLocation("test.yml"),
         ).to_plain()
 
 
+def test_sigmadetection_with_mixed_items_to_plain():
+    with pytest.raises(sigma_exceptions.SigmaValueError, match="Can't convert detection.*test.yml"):
+        SigmaDetection(
+            detection_items=[
+                SigmaDetectionItem("field1", [], [SigmaString("value1")]),
+                SigmaDetection(
+                    [SigmaDetectionItem("field2", [], [SigmaString("value2")])],
+                ),
+            ],
+            source=sigma_exceptions.SigmaRuleLocation("test.yml"),
+        ).to_plain()
+
+
+def test_sigmadetection_list_of_sigmadetection_to_plain():
+    assert SigmaDetection(
+        detection_items=[
+            SigmaDetection([SigmaDetectionItem("field1", [], [SigmaString("value1")])]),
+            SigmaDetection([SigmaDetectionItem("field2", [], [SigmaString("value2")])]),
+        ]
+    ).to_plain() == [{"field1": "value1"}, {"field2": "value2"}]
+
+
 ### SigmaRule tests ###
 
 
@@ -879,15 +1059,68 @@ def test_sigmarule_bad_status_type():
 
 
 def test_sigmarule_bad_date():
-    with pytest.raises(sigma_exceptions.SigmaDateError, match="Rule date.*test.yml"):
-        SigmaRule.from_dict({"date": "bad"}, source=sigma_exceptions.SigmaRuleLocation("test.yml"))
+    """This test uses string data type as date representation in yaml"""
+    bad_string_dates = (
+        "bad",
+        " 2024-11-24",
+        "2024-11-24 ",
+        "2024 11-24",
+        "24-11-24",
+        "02-02-02",
+        "4000-01-01",
+        "10000-01-01",
+        "2022-01/01",
+        "2022/01-01",
+    )
+    for test_string in bad_string_dates:
+        match_string = f"Rule date '{test_string}' is invalid, use yyyy-mm-dd"
+        with pytest.raises(sigma_exceptions.SigmaDateError, match=match_string) as ex:
+            SigmaRule.from_yaml(
+                f"""
+                title: Test
+                date: '{test_string}'  # try a string
+                logsource:
+                    product: foobar
+                detection:
+                    selection_1:
+                        fieldA: valueA
+                    condition: selection_1
+                """
+            )
+            assert False, f"Did not throw SigmaDateError on date {test_string}"
 
 
 def test_sigmarule_bad_modified():
-    with pytest.raises(sigma_exceptions.SigmaModifiedError, match="Rule modified.*test.yml"):
-        SigmaRule.from_dict(
-            {"modified": "bad"}, source=sigma_exceptions.SigmaRuleLocation("test.yml")
-        )
+    """
+    This test uses yaml ability to recognize dates.
+    Therefore, here 4000-01-01 will be interpreted as a correct yaml date.
+    """
+    bad_dates = (
+        "bad",
+        "24-11-24",
+        "02-02-02",
+        "2024-5-5",
+        "10000-01-01",
+        "2022-01/01",
+        "2022/01-01",
+        "2022 01/01",
+    )
+    for test_string in bad_dates:
+        match_string = f"Rule modified '{test_string}' is invalid, use yyyy-mm-dd"
+        with pytest.raises(sigma_exceptions.SigmaModifiedError, match=match_string) as ex:
+            SigmaRule.from_yaml(
+                f"""
+                title: Test
+                modified: {test_string}  # this can be recognized as date by yaml parser
+                logsource:
+                    product: foobar
+                detection:
+                    selection_1:
+                        fieldA: valueA
+                    condition: selection_1
+                """
+            )
+            assert False, f"Did not throw SigmaModifiedError on date {test_string}"
 
 
 def test_sigmarule_bad_falsepositives():
@@ -932,25 +1165,36 @@ def test_sigmarule_date():
 
 
 def test_modified_date():
-    expected_date = date(3000, 11, 3)
-    rule = SigmaRule.from_yaml(
-        """
-    title: Test
-    id: cafedead-beef-0000-1111-0123456789ab
-    level: medium
-    status: test
-    date: 3000-01-02
-    modified: 3000-11-03
-    logsource:
-        product: foobar
-    detection:
-        selection_1:
-            fieldA: valueA
-        condition: selection_1
-    """
-    )
-    assert rule is not None
-    assert rule.modified == expected_date
+    validDates = {
+        "3000-12-31": date(3000, 12, 31),  # can appear as a generic date in the future
+        "2999-04-04": date(2999, 4, 4),
+        "2024-11-22": date(2024, 11, 22),
+        "1970-01-01": date(1970, 1, 1),  # can appear as a generic date in the past
+        "1900-12-31": date(1900, 12, 31),  # not much useful, but correct
+        "2024/11/22": date(2024, 11, 22),  # US-based sigmas have such dates
+        "2024/1/2": date(2024, 1, 2),
+        "1970/1/1": date(1970, 1, 1),
+    }
+    for test_string, expected_date in validDates.items():
+        rule = SigmaRule.from_yaml(
+            f"""
+            title: Test
+            id: cafedead-beef-0000-1111-0123456789ab
+            level: medium
+            status: test
+            date: {test_string}       # possibly a yaml date
+            modified: '{test_string}' # always a string data type converted into date
+            logsource:
+                product: foobar
+            detection:
+                selection_1:
+                    fieldA: valueA
+                condition: selection_1
+            """
+        )
+        assert rule is not None
+        assert rule.date == expected_date, f"bad 'date' for '{test_string}'"
+        assert rule.modified == expected_date, f"bad 'modified' for '{test_string}'"
 
 
 def test_sigmarule_datetime():
@@ -1086,14 +1330,16 @@ def sigma_rule():
                         SigmaDetection(
                             [
                                 SigmaDetectionItem(
-                                    "CommandLine", [SigmaContainsModifier], ["test.exe"]
+                                    "CommandLine",
+                                    [SigmaContainsModifier],
+                                    [SigmaString("test.exe")],
                                 )
                             ]
                         ),
                         SigmaDetection(
                             [
                                 SigmaDetectionItem(
-                                    "CommandLine", [SigmaContainsModifier], ["cmd.exe"]
+                                    "CommandLine", [SigmaContainsModifier], [SigmaString("cmd.exe")]
                                 )
                             ]
                         ),
@@ -1101,7 +1347,9 @@ def sigma_rule():
                 ),
                 "selection_3": SigmaDetection(
                     [
-                        SigmaDetectionItem(None, [], ["keyword_1", "keyword_2"]),
+                        SigmaDetectionItem(
+                            None, [], [SigmaString("keyword_1"), SigmaString("keyword_2")]
+                        ),
                     ]
                 ),
             },
@@ -1115,6 +1363,10 @@ def sigma_rule():
             "Everything",
         ],
         level=SigmaLevel.LOW,
+        scope=[
+            "scope1",
+            "scope2",
+        ],
     )
 
 
@@ -1154,6 +1406,9 @@ def test_sigmarule_fromyaml(sigma_rule):
     falsepositives:
         - Everything
     level: low
+    scope:
+        - scope1
+        - scope2     
     """
     )
     assert sigmarule_from_yaml == sigma_rule
@@ -1196,6 +1451,9 @@ def test_sigmarule_fromyaml_with_custom_attribute(sigma_rule):
         - Everything
     level: low
     custom: attribute
+    scope:
+        - scope1
+        - scope2    
     """
     )
     assert sigmarule_from_yaml == sigma_rule
@@ -1248,9 +1506,10 @@ def test_sigmarule_to_dict(sigma_rule: SigmaRule):
             "selection_1": {
                 "CommandLine|contains": "test.exe",
             },
-            "selection_2": {
-                "CommandLine|contains|all": ["test.exe", "cmd.exe"],
-            },
+            "selection_2": [
+                {"CommandLine|contains": "test.exe"},
+                {"CommandLine|contains": "cmd.exe"},
+            ],
             "selection_3": [
                 "keyword_1",
                 "keyword_2",
@@ -1262,6 +1521,7 @@ def test_sigmarule_to_dict(sigma_rule: SigmaRule):
             "Everything",
         ],
         "level": "low",
+        "scope": ["scope1", "scope2"],
     }
 
 
@@ -1548,3 +1808,66 @@ def test_sigmarule_bad_scope():
             {"title": "test", "scope": "windows AD"},
             source=sigma_exceptions.SigmaRuleLocation("test.yml"),
         )
+
+
+def test_sigmarule_timestamp_modifiers_equals():
+    rule = SigmaRule.from_dict(
+        {
+            "title": "Test",
+            "logsource": {
+                "category": "process_creation",
+                "product": "windows",
+            },
+            "detection": {
+                "selection": {
+                    "timestamp|minute": 1,
+                    "timestamp|hour": 2,
+                    "timestamp|day": 3,
+                    "timestamp|week": 4,
+                    "timestamp|month": 5,
+                    "timestamp|year": 6,
+                },
+                "condition": "selection",
+            },
+        },
+        source=sigma_exceptions.SigmaRuleLocation("test.yml"),
+    )
+    detection_items = rule.detection["selection"].detection_items
+    assert detection_items[0].value[0] == SigmaTimestampPart(TimestampPart.MINUTE, 1)
+    assert detection_items[1].value[0] == SigmaTimestampPart(TimestampPart.HOUR, 2)
+    assert detection_items[2].value[0] == SigmaTimestampPart(TimestampPart.DAY, 3)
+    assert detection_items[3].value[0] == SigmaTimestampPart(TimestampPart.WEEK, 4)
+    assert detection_items[4].value[0] == SigmaTimestampPart(TimestampPart.MONTH, 5)
+    assert detection_items[5].value[0] == SigmaTimestampPart(TimestampPart.YEAR, 6)
+
+
+def test_sigmarule_timestamp_modifiers_greater_than():
+    rule = SigmaRule.from_dict(
+        {
+            "title": "Test",
+            "logsource": {
+                "category": "process_creation",
+                "product": "windows",
+            },
+            "detection": {
+                "selection": {
+                    "timestamp|minute|gt": 1,
+                    "timestamp|hour|gte": 2,
+                    "timestamp|day|lt": 3,
+                    "timestamp|week|lte": 4,
+                    "timestamp|month|gt": 5,
+                    "timestamp|year|gte": 6,
+                },
+                "condition": "selection",
+            },
+        },
+        source=sigma_exceptions.SigmaRuleLocation("test.yml"),
+    )
+    detection_items = rule.detection["selection"].detection_items
+    assert isinstance(detection_items[0].value[0], SigmaCompareExpression)
+    assert detection_items[0].value[0].number == SigmaTimestampPart(TimestampPart.MINUTE, 1)
+    assert detection_items[1].value[0].number == SigmaTimestampPart(TimestampPart.HOUR, 2)
+    assert detection_items[2].value[0].number == SigmaTimestampPart(TimestampPart.DAY, 3)
+    assert detection_items[3].value[0].number == SigmaTimestampPart(TimestampPart.WEEK, 4)
+    assert detection_items[4].value[0].number == SigmaTimestampPart(TimestampPart.MONTH, 5)
+    assert detection_items[5].value[0].number == SigmaTimestampPart(TimestampPart.YEAR, 6)
