@@ -10,6 +10,7 @@ from sigma.exceptions import (
     SigmaRegularExpressionError,
 )
 from sigma.types import (
+    CompareOperators,
     SigmaBool,
     SigmaCasedString,
     SigmaCompareExpression,
@@ -38,83 +39,83 @@ def empty_sigma_string():
 
 
 def test_strings_empty():
-    assert SigmaString().s == tuple()
+    assert SigmaString().s == []
 
 
 def test_strings_plain():
-    assert SigmaString("plain").s == ("plain",)
+    assert SigmaString("plain").s == ["plain"]
 
 
 def test_strings_merge():
     s = SigmaString()
-    s.s = (SpecialChars.WILDCARD_MULTI, "te", "st", SpecialChars.WILDCARD_MULTI)
-    assert s._merge_strs().s == (
+    s.s = [SpecialChars.WILDCARD_MULTI, "te", "st", SpecialChars.WILDCARD_MULTI]
+    assert s._merge_strs().s == [
         SpecialChars.WILDCARD_MULTI,
         "test",
         SpecialChars.WILDCARD_MULTI,
-    )
+    ]
 
 
 def test_strings_merge_end():
     s = SigmaString()
-    s.s = (
+    s.s = [
         SpecialChars.WILDCARD_MULTI,
         "test",
         SpecialChars.WILDCARD_MULTI,
         "te",
         "st",
         "test",
-    )
-    assert s._merge_strs().s == (
+    ]
+    assert s._merge_strs().s == [
         SpecialChars.WILDCARD_MULTI,
         "test",
         SpecialChars.WILDCARD_MULTI,
         "testtest",
-    )
+    ]
 
 
 def test_strings_merge_start():
     s = SigmaString()
-    s.s = (
+    s.s = [
         "te",
         "st",
         "test",
         SpecialChars.WILDCARD_MULTI,
         "test",
         SpecialChars.WILDCARD_MULTI,
-    )
-    assert s._merge_strs().s == (
+    ]
+    assert s._merge_strs().s == [
         "testtest",
         SpecialChars.WILDCARD_MULTI,
         "test",
         SpecialChars.WILDCARD_MULTI,
-    )
+    ]
 
 
 def test_strings_wildcards():
-    assert SigmaString("wild*cards?contained").s == (
+    assert SigmaString("wild*cards?contained").s == [
         "wild",
         SpecialChars.WILDCARD_MULTI,
         "cards",
         SpecialChars.WILDCARD_SINGLE,
         "contained",
-    )
+    ]
 
 
 def test_strings_escaping():
-    assert SigmaString("escaped\\*\\?\\\\*?").s == (
+    assert SigmaString("escaped\\*\\?\\\\*?").s == [
         "escaped*?\\",
         SpecialChars.WILDCARD_MULTI,
         SpecialChars.WILDCARD_SINGLE,
-    )
+    ]
 
 
 def test_strings_escaping_nonspecial():
-    assert SigmaString("escaped\\nonspecial").s == ("escaped\\nonspecial",)
+    assert SigmaString("escaped\\nonspecial").s == ["escaped\\nonspecial"]
 
 
 def test_strings_escaping_end():
-    assert SigmaString("finalescape\\").s == ("finalescape\\",)
+    assert SigmaString("finalescape\\").s == ["finalescape\\"]
 
 
 def test_strings_from_str():
@@ -122,15 +123,15 @@ def test_strings_from_str():
 
 
 def test_string_placeholders_single():
-    assert SigmaString("test1%var%test2").insert_placeholders().s == (
+    assert SigmaString("test1%var%test2").insert_placeholders().s == [
         "test1",
         Placeholder("var"),
         "test2",
-    )
+    ]
 
 
 def test_string_placeholders_multi():
-    assert SigmaString("%start%te*st1%middle%te?st2%end%").insert_placeholders().s == (
+    assert SigmaString("%start%te*st1%middle%te?st2%end%").insert_placeholders().s == [
         Placeholder("start"),
         "te",
         SpecialChars.WILDCARD_MULTI,
@@ -140,13 +141,13 @@ def test_string_placeholders_multi():
         SpecialChars.WILDCARD_SINGLE,
         "st2",
         Placeholder("end"),
-    )
+    ]
 
 
 def test_string_replace_with_placeholder():
     assert SigmaString("testx1xfoox1xtest*x2x*bar*x3x").replace_with_placeholder(
         re.compile("x\\dx"), "test"
-    ).s == (
+    ).s == [
         "test",
         Placeholder("test"),
         "foo",
@@ -158,7 +159,7 @@ def test_string_replace_with_placeholder():
         "bar",
         SpecialChars.WILDCARD_MULTI,
         Placeholder("test"),
-    )
+    ]
 
 
 def test_string_placeholders_replace():
@@ -176,11 +177,11 @@ def test_string_placeholders_replace():
 
 
 def test_string_placeholders_escape():
-    assert SigmaString("\\%test1\\%test2\\%%var%\\%test3\\%").insert_placeholders().s == (
+    assert SigmaString("\\%test1\\%test2\\%%var%\\%test3\\%").insert_placeholders().s == [
         "%test1%test2%",
         Placeholder("var"),
         "%test3%",
-    )
+    ]
 
 
 def test_string_contains_placeholders():
@@ -323,13 +324,13 @@ def test_strings_to_string():
 def test_strings_with_plain_wildcards_to_string():
     plain_s = r"value\*with\?plain*wild?cards"
     s = SigmaString(plain_s)
-    assert s.s == (
+    assert s.s == [
         "value*with?plain",
         SpecialChars.WILDCARD_MULTI,
         "wild",
         SpecialChars.WILDCARD_SINGLE,
         "cards",
-    )
+    ]
     assert str(s) == plain_s
     assert SigmaString(str(s)) == s
 
@@ -383,28 +384,28 @@ def test_strings_convert_placeholder():
 
 def test_strings_convert_invalid_part():
     s = SigmaString("test")
-    s.s = ("test", 1)
+    s.s = ["test", 1]
     with pytest.raises(SigmaValueError, match="part of type 'int'"):
         s.convert()
 
 
 def test_strings_to_regex():
     s = SigmaString("Test*Special?(Plain)/[\\*\\?]")
-    assert s.s == (
+    assert s.s == [
         "Test",
         SpecialChars.WILDCARD_MULTI,
         "Special",
         SpecialChars.WILDCARD_SINGLE,
         "(Plain)/[*?]",
-    )
+    ]
     r = s.to_regex()
-    assert r.regexp == "Test.*Special.\\(Plain\\)/\\[\\*\\?\\]"
+    assert str(r.regexp) == "Test.*Special.\\(Plain\\)/\\[\\*\\?\\]"
 
 
 def test_strings_to_regex_with_additional_escape_chars():
     s = SigmaString("Test*Special?(Plain)/[\\*\\?]")
     r = s.to_regex("/")
-    assert r.regexp == "Test.*Special.\\(Plain\\)\\/\\[\\*\\?\\]"
+    assert str(r.regexp) == "Test.*Special.\\(Plain\\)\\/\\[\\*\\?\\]"
 
 
 def test_string_index(sigma_string):
@@ -467,6 +468,14 @@ def test_string_index_slice_open_start_negative_end(sigma_string):
     assert sigma_string[:-1] == SigmaString("*Test*Str\\*ing")
 
 
+def test_string_index_cased_index():
+    assert SigmaCasedString("TestCasedString")[4:9] == SigmaCasedString("Cased")
+
+
+def test_string_index_cased_type():
+    assert type(SigmaCasedString("TestCasedString")[4:9]) is SigmaCasedString
+
+
 def test_string_index_invalid_type(sigma_string):
     with pytest.raises(TypeError, match="indices must be"):
         sigma_string["invalid"]
@@ -488,17 +497,17 @@ def test_string_iter_parts(sigma_string):
 
 
 def test_string_map_parts(sigma_string):
-    assert sigma_string.map_parts(lambda x: x.upper(), lambda x: isinstance(x, str)).s == (
+    assert sigma_string.map_parts(lambda x: x.upper(), lambda x: isinstance(x, str)).s == [
         SpecialChars.WILDCARD_MULTI,
         "TEST",
         SpecialChars.WILDCARD_MULTI,
         "STR*ING",
         SpecialChars.WILDCARD_MULTI,
-    )
+    ]
 
 
 def test_string_map_parts_interpret_special(sigma_string):
-    assert sigma_string.map_parts(lambda x: x.upper(), lambda x: isinstance(x, str), True).s == (
+    assert sigma_string.map_parts(lambda x: x.upper(), lambda x: isinstance(x, str), True).s == [
         SpecialChars.WILDCARD_MULTI,
         "TEST",
         SpecialChars.WILDCARD_MULTI,
@@ -506,7 +515,7 @@ def test_string_map_parts_interpret_special(sigma_string):
         SpecialChars.WILDCARD_MULTI,
         "ING",
         SpecialChars.WILDCARD_MULTI,
-    )
+    ]
 
 
 def test_cased_string(sigma_string):
@@ -529,10 +538,23 @@ def test_number_equal():
     assert SigmaNumber(123) == SigmaNumber(123)
 
 
+def test_number_not_equal():
+    assert SigmaNumber(123) != SigmaNumber(456)
+
+
 def test_number_equal_plain():
     assert SigmaNumber(123) == 123
 
 
+def test_number_not_equal_plain():
+    assert SigmaNumber(123) != 456
+
+
+def test_number_equal_wrong_type():
+    with pytest.raises(NotImplementedError, match="SigmaNumber can only be compared"):
+        SigmaNumber(123) == "123"
+
+
 def test_number_invalid():
     with pytest.raises(SigmaValueError):
         SigmaNumber("test")
@@ -629,6 +651,27 @@ def test_bool_invalid():
         SigmaBool(123)
 
 
+def test_bool_equal():
+    assert SigmaBool(True) == SigmaBool(True)
+
+
+def test_bool_not_equal():
+    assert SigmaBool(True) != SigmaBool(False)
+
+
+def test_bool_equal_plain():
+    assert SigmaBool(True) == True
+
+
+def test_bool_not_equal_plain():
+    assert SigmaBool(True) != False
+
+
+def test_bool_equal_wrong_type():
+    with pytest.raises(NotImplementedError, match="SigmaBool can only be compared"):
+        SigmaBool(True) == "True"
+
+
 def test_null_to_plain():
     assert SigmaNull().to_plain() == None
 
@@ -661,6 +704,11 @@ def test_conversion_none():
     assert sigma_type(None) == SigmaNull()
 
 
+def test_conversion_unsupported():
+    with pytest.raises(SigmaTypeError, match="Unsupported type"):
+        sigma_type(object())
+
+
 def test_query_expression():
     assert str(SigmaQueryExpression("test\\test*test?test[]", "id")) == "test\\test*test?test[]"
 
@@ -704,6 +752,10 @@ def test_cidr_to_plain():
         SigmaCIDRExpression("192.168.1.0/24").to_plain()
 
 
+def test_cidr_to_str():
+    assert str(SigmaCIDRExpression("192.168.1.0/24")) == "192.168.1.0/24"
+
+
 def test_cidr_invalid():
     with pytest.raises(SigmaTypeError, match="Invalid CIDR expression"):
         SigmaCIDRExpression("1/132")
@@ -788,13 +840,9 @@ def test_cidr_invalid():
 
 def test_compare_to_plain():
     with pytest.raises(SigmaValueError, match="can't be converted into a plain representation"):
-        SigmaCompareExpression(
-            SigmaNumber(123), SigmaCompareExpression.CompareOperators.LTE
-        ).to_plain()
+        SigmaCompareExpression(SigmaNumber(123), CompareOperators.LTE).to_plain()
 
 
 def test_compare_string():
     with pytest.raises(SigmaTypeError, match="expects number"):
-        SigmaCompareExpression(
-            SigmaString("123"), SigmaCompareExpression.CompareOperators.LTE
-        ).to_plain()
+        SigmaCompareExpression(SigmaString("123"), CompareOperators.LTE).to_plain()
